package jdk.internal.org.objectweb.asm.util;

import jdk.internal.org.objectweb.asm.TypeReference;
import java.util.HashMap;
import jdk.internal.org.objectweb.asm.Handle;
import jdk.internal.org.objectweb.asm.Type;
import jdk.internal.org.objectweb.asm.Attribute;
import jdk.internal.org.objectweb.asm.TypePath;
import jdk.internal.org.objectweb.asm.signature.SignatureReader;
import java.io.PrintWriter;
import java.io.FileInputStream;
import jdk.internal.org.objectweb.asm.ClassReader;
import jdk.internal.org.objectweb.asm.Label;
import java.util.Map;

public class Textifier extends Printer
{
    public static final int INTERNAL_NAME = 0;
    public static final int FIELD_DESCRIPTOR = 1;
    public static final int FIELD_SIGNATURE = 2;
    public static final int METHOD_DESCRIPTOR = 3;
    public static final int METHOD_SIGNATURE = 4;
    public static final int CLASS_SIGNATURE = 5;
    public static final int TYPE_DECLARATION = 6;
    public static final int CLASS_DECLARATION = 7;
    public static final int PARAMETERS_DECLARATION = 8;
    public static final int HANDLE_DESCRIPTOR = 9;
    protected String tab;
    protected String tab2;
    protected String tab3;
    protected String ltab;
    protected Map<Label, String> labelNames;
    private int access;
    private int valueNumber;
    
    public Textifier() {
        this(327680);
        if (this.getClass() != Textifier.class) {
            throw new IllegalStateException();
        }
        return;
    }
    
    protected Textifier(int n) {
        super(n);
        this.tab = "  ";
        this.tab2 = "    ";
        this.tab3 = "      ";
        this.ltab = "   ";
        this.valueNumber = 0;
    }
    
    public static void main(String[] array) throws Exception {
        int n = 0;
        int parsingOptions = 2;
        int n2 = 1;
        if (array.length < 1 || array.length > 2) {
            n2 = 0;
        }
        if (n2 != 0 && "-debug".equals(array[0])) {
            n = 1;
            parsingOptions = 0;
            if (array.length != 2) {
                n2 = 0;
            }
        }
        if (n2 == 0) {
            System.err.println("Prints a disassembled view of the given class.");
            System.err.println("Usage: Textifier [-debug] <fully qualified class name or class file name>");
            return;
        }
        Object o;
        if (array[n].endsWith(".class") || array[n].indexOf(92) > -1 || array[n].indexOf(47) > -1) {
            o = new ClassReader(new FileInputStream(array[n]));
        }
        else {
            o = new ClassReader(array[n]);
        }
        o.accept(new TraceClassVisitor(new PrintWriter(System.out)), parsingOptions);
        return;
    }
    
    @Override
    public void visit(int i, int n, String str, String signature, String anObject, String[] array) {
        this.access = n;
        int j = i & 0xFFFF;
        int k = i >>> 16;
        this.buf.setLength(0);
        this.buf.append("// class version ").append(j).append('.').append(k).append(" (").append(i).append(")\n");
        if ((n & 0x20000) != 0x0) {
            this.buf.append("// DEPRECATED\n");
        }
        this.buf.append("// access flags 0x").append(Integer.toHexString(n).toUpperCase()).append('\n');
        this.appendDescriptor(5, signature);
        if (signature != null) {
            TraceSignatureVisitor signatureVistor = new TraceSignatureVisitor(n);
            new SignatureReader(signature).accept(signatureVistor);
            this.buf.append("// declaration: ").append(str).append(signatureVistor.getDeclaration()).append('\n');
        }
        this.appendAccess(n & 0xFFFFFFDF);
        if ((n & 0x2000) != 0x0) {
            this.buf.append("@interface ");
        }
        else if ((n & 0x200) != 0x0) {
            this.buf.append("interface ");
        }
        else if ((n & 0x4000) == 0x0) {
            this.buf.append("class ");
        }
        this.appendDescriptor(0, str);
        if (anObject != null && !"java/lang/Object".equals(anObject)) {
            this.buf.append(" extends ");
            this.appendDescriptor(0, anObject);
            this.buf.append(' ');
        }
        if (array != null && array.length > 0) {
            this.buf.append(" implements ");
            for (int l = 0; l < array.length; ++l) {
                this.appendDescriptor(0, array[l]);
                this.buf.append(' ');
            }
        }
        this.buf.append(" {\n\n");
        this.text.add(this.buf.toString());
    }
    
    @Override
    public void visitSource(String str, String str2) {
        this.buf.setLength(0);
        if (str != null) {
            this.buf.append(this.tab).append("// compiled from: ").append(str).append('\n');
        }
        if (str2 != null) {
            this.buf.append(this.tab).append("// debug info: ").append(str2).append('\n');
        }
        if (this.buf.length() > 0) {
            this.text.add(this.buf.toString());
        }
    }
    
    @Override
    public void visitOuterClass(String s, String str, String s2) {
        this.buf.setLength(0);
        this.buf.append(this.tab).append("OUTERCLASS ");
        this.appendDescriptor(0, s);
        this.buf.append(' ');
        if (str != null) {
            this.buf.append(str).append(' ');
        }
        this.appendDescriptor(3, s2);
        this.buf.append('\n');
        this.text.add(this.buf.toString());
    }
    
    @Override
    public Textifier visitClassAnnotation(String s, boolean b) {
        this.text.add("\n");
        return this.visitAnnotation(s, b);
    }
    
    @Override
    public Printer visitClassTypeAnnotation(int n, TypePath typePath, String s, boolean b) {
        this.text.add("\n");
        return this.visitTypeAnnotation(n, typePath, s, b);
    }
    
    @Override
    public void visitClassAttribute(Attribute attribute) {
        this.text.add("\n");
        this.visitAttribute(attribute);
    }
    
    @Override
    public void visitInnerClass(String s, String s2, String s3, int n) {
        this.buf.setLength(0);
        this.buf.append(this.tab).append("// access flags 0x");
        this.buf.append(Integer.toHexString(n & 0xFFFFFFDF).toUpperCase()).append('\n');
        this.buf.append(this.tab);
        this.appendAccess(n);
        this.buf.append("INNERCLASS ");
        this.appendDescriptor(0, s);
        this.buf.append(' ');
        this.appendDescriptor(0, s2);
        this.buf.append(' ');
        this.appendDescriptor(0, s3);
        this.buf.append('\n');
        this.text.add(this.buf.toString());
    }
    
    @Override
    public Textifier visitField(int i, String str, String s, String signature, Object o) {
        this.buf.setLength(0);
        this.buf.append('\n');
        if ((i & 0x20000) != 0x0) {
            this.buf.append(this.tab).append("// DEPRECATED\n");
        }
        this.buf.append(this.tab).append("// access flags 0x").append(Integer.toHexString(i).toUpperCase()).append('\n');
        if (signature != null) {
            this.buf.append(this.tab);
            this.appendDescriptor(2, signature);
            TraceSignatureVisitor signatureVisitor = new TraceSignatureVisitor(0);
            new SignatureReader(signature).acceptType(signatureVisitor);
            this.buf.append(this.tab).append("// declaration: ").append(signatureVisitor.getDeclaration()).append('\n');
        }
        this.buf.append(this.tab);
        this.appendAccess(i);
        this.appendDescriptor(1, s);
        this.buf.append(' ').append(str);
        if (o != null) {
            this.buf.append(" = ");
            if ((o instanceof String)) {
                this.buf.append('\"').append(o).append('\"');
            }
            else {
                this.buf.append(o);
            }
        }
        this.buf.append('\n');
        this.text.add(this.buf.toString());
        Textifier textifier = this.createTextifier();
        this.text.add(textifier.getText());
        return textifier;
    }
    
    @Override
    public Textifier visitMethod(int i, String s, String s2, String signature, String[] array) {
        this.buf.setLength(0);
        this.buf.append('\n');
        if ((i & 0x20000) != 0x0) {
            this.buf.append(this.tab).append("// DEPRECATED\n");
        }
        this.buf.append(this.tab).append("// access flags 0x").append(Integer.toHexString(i).toUpperCase()).append('\n');
        if (signature != null) {
            this.buf.append(this.tab);
            this.appendDescriptor(4, signature);
            TraceSignatureVisitor signatureVistor = new TraceSignatureVisitor(0);
            new SignatureReader(signature).accept(signatureVistor);
            String declaration = signatureVistor.getDeclaration();
            String returnType = signatureVistor.getReturnType();
            String exceptions = signatureVistor.getExceptions();
            this.buf.append(this.tab).append("// declaration: ").append(returnType).append(' ').append(s).append(declaration);
            if (exceptions != null) {
                this.buf.append(" throws ").append(exceptions);
            }
            this.buf.append('\n');
        }
        this.buf.append(this.tab);
        this.appendAccess(i & 0xFFFFFFBF);
        if ((i & 0x100) != 0x0) {
            this.buf.append("native ");
        }
        if ((i & 0x80) != 0x0) {
            this.buf.append("varargs ");
        }
        if ((i & 0x40) != 0x0) {
            this.buf.append("bridge ");
        }
        if ((this.access & 0x200) != 0x0 && (i & 0x400) == 0x0 && (i & 0x8) == 0x0) {
            this.buf.append("default ");
        }
        this.buf.append(s);
        this.appendDescriptor(3, s2);
        if (array != null && array.length > 0) {
            this.buf.append(" throws ");
            for (int j = 0; j < array.length; ++j) {
                this.appendDescriptor(0, array[j]);
                this.buf.append(' ');
            }
        }
        this.buf.append('\n');
        this.text.add(this.buf.toString());
        Textifier textifier = this.createTextifier();
        this.text.add(textifier.getText());
        return textifier;
    }
    
    @Override
    public void visitClassEnd() {
        this.text.add("}\n");
    }
    
    @Override
    public void visit(String str, Object o) {
        this.buf.setLength(0);
        this.appendComa(this.valueNumber++);
        if (str != null) {
            this.buf.append(str).append('=');
        }
        if ((o instanceof String)) {
            this.visitString((String)o);
        }
        else if ((o instanceof Type)) {
            this.visitType((Type)o);
        }
        else if ((o instanceof Byte)) {
            this.visitByte((byte)o);
        }
        else if ((o instanceof Boolean)) {
            this.visitBoolean((boolean)o);
        }
        else if ((o instanceof Short)) {
            this.visitShort((short)o);
        }
        else if ((o instanceof Character)) {
            this.visitChar((char)o);
        }
        else if ((o instanceof Integer)) {
            this.visitInt((int)o);
        }
        else if ((o instanceof Float)) {
            this.visitFloat((float)o);
        }
        else if ((o instanceof Long)) {
            this.visitLong((long)o);
        }
        else if ((o instanceof Double)) {
            this.visitDouble((double)o);
        }
        else if (o.getClass().isArray()) {
            this.buf.append('{');
            if ((o instanceof byte[])) {
                byte[] array8 = (byte[])(byte[])o;
                for (int n4 = 0; n4 < array8.length; ++n4) {
                    this.appendComa(n4);
                    this.visitByte(array8[n4]);
                }
            }
            else if ((o instanceof boolean[])) {
                boolean[] array7 = (boolean[])(boolean[])o;
                for (int n3 = 0; n3 < array7.length; ++n3) {
                    this.appendComa(n3);
                    this.visitBoolean(array7[n3]);
                }
            }
            else if ((o instanceof short[])) {
                short[] array6 = (short[])(short[])o;
                for (int n2 = 0; n2 < array6.length; ++n2) {
                    this.appendComa(n2);
                    this.visitShort(array6[n2]);
                }
            }
            else if ((o instanceof char[])) {
                char[] array5 = (char[])(char[])o;
                for (int n = 0; n < array5.length; ++n) {
                    this.appendComa(n);
                    this.visitChar(array5[n]);
                }
            }
            else if ((o instanceof int[])) {
                int[] array4 = (int[])(int[])o;
                for (int l = 0; l < array4.length; ++l) {
                    this.appendComa(l);
                    this.visitInt(array4[l]);
                }
            }
            else if ((o instanceof long[])) {
                long[] array3 = (long[])(long[])o;
                for (int k = 0; k < array3.length; ++k) {
                    this.appendComa(k);
                    this.visitLong(array3[k]);
                }
            }
            else if ((o instanceof float[])) {
                float[] array2 = (float[])(float[])o;
                for (int j = 0; j < array2.length; ++j) {
                    this.appendComa(j);
                    this.visitFloat(array2[j]);
                }
            }
            else if ((o instanceof double[])) {
                double[] array = (double[])(double[])o;
                for (int i = 0; i < array.length; ++i) {
                    this.appendComa(i);
                    this.visitDouble(array[i]);
                }
            }
            this.buf.append('}');
        }
        this.text.add(this.buf.toString());
    }
    
    private void visitInt(int i) {
        this.buf.append(i);
    }
    
    private void visitLong(long lng) {
        this.buf.append(lng).append('L');
    }
    
    private void visitFloat(float f) {
        this.buf.append(f).append('F');
    }
    
    private void visitDouble(double d) {
        this.buf.append(d).append('D');
    }
    
    private void visitChar(char i) {
        this.buf.append("(char)").append((int)i);
    }
    
    private void visitShort(short i) {
        this.buf.append("(short)").append((int)i);
    }
    
    private void visitByte(byte i) {
        this.buf.append("(byte)").append((int)i);
    }
    
    private void visitBoolean(boolean b) {
        this.buf.append(b);
    }
    
    private void visitString(String s) {
        Printer.appendString(this.buf, s);
    }
    
    private void visitType(Type type) {
        this.buf.append(type.getClassName()).append(".class");
    }
    
    @Override
    public void visitEnum(String str, String s, String str2) {
        this.buf.setLength(0);
        this.appendComa(this.valueNumber++);
        if (str != null) {
            this.buf.append(str).append('=');
        }
        this.appendDescriptor(1, s);
        this.buf.append('.').append(str2);
        this.text.add(this.buf.toString());
    }
    
    @Override
    public Textifier visitAnnotation(String str, String s) {
        this.buf.setLength(0);
        this.appendComa(this.valueNumber++);
        if (str != null) {
            this.buf.append(str).append('=');
        }
        this.buf.append('@');
        this.appendDescriptor(1, s);
        this.buf.append('(');
        this.text.add(this.buf.toString());
        Textifier textifier = this.createTextifier();
        this.text.add(textifier.getText());
        this.text.add(")");
        return textifier;
    }
    
    @Override
    public Textifier visitArray(String str) {
        this.buf.setLength(0);
        this.appendComa(this.valueNumber++);
        if (str != null) {
            this.buf.append(str).append('=');
        }
        this.buf.append('{');
        this.text.add(this.buf.toString());
        Textifier textifier = this.createTextifier();
        this.text.add(textifier.getText());
        this.text.add("}");
        return textifier;
    }
    
    @Override
    public void visitAnnotationEnd() {
    }
    
    @Override
    public Textifier visitFieldAnnotation(String s, boolean b) {
        return this.visitAnnotation(s, b);
    }
    
    @Override
    public Printer visitFieldTypeAnnotation(int n, TypePath typePath, String s, boolean b) {
        return this.visitTypeAnnotation(n, typePath, s, b);
    }
    
    @Override
    public void visitFieldAttribute(Attribute attribute) {
        this.visitAttribute(attribute);
    }
    
    @Override
    public void visitFieldEnd() {
    }
    
    @Override
    public void visitParameter(String s, int n) {
        this.buf.setLength(0);
        this.buf.append(this.tab2).append("// parameter ");
        this.appendAccess(n);
        this.buf.append(' ').append((s == null) ? "<no name>" : s).append('\n');
        this.text.add(this.buf.toString());
    }
    
    @Override
    public Textifier visitAnnotationDefault() {
        this.text.add(this.tab2 + "default=");
        Textifier textifier = this.createTextifier();
        this.text.add(textifier.getText());
        this.text.add("\n");
        return textifier;
    }
    
    @Override
    public Textifier visitMethodAnnotation(String s, boolean b) {
        return this.visitAnnotation(s, b);
    }
    
    @Override
    public Printer visitMethodTypeAnnotation(int n, TypePath typePath, String s, boolean b) {
        return this.visitTypeAnnotation(n, typePath, s, b);
    }
    
    @Override
    public Textifier visitParameterAnnotation(int i, String s, boolean b) {
        this.buf.setLength(0);
        this.buf.append(this.tab2).append('@');
        this.appendDescriptor(1, s);
        this.buf.append('(');
        this.text.add(this.buf.toString());
        Textifier textifier = this.createTextifier();
        this.text.add(textifier.getText());
        this.text.add(b ? ") // parameter " : ") // invisible, parameter ");
        this.text.add(Integer.valueOf(i));
        this.text.add("\n");
        return textifier;
    }
    
    @Override
    public void visitMethodAttribute(Attribute attribute) {
        this.buf.setLength(0);
        this.buf.append(this.tab).append("ATTRIBUTE ");
        this.appendDescriptor(-1, attribute.type);
        if ((attribute instanceof Textifiable)) {
            ((Textifiable)attribute).textify(this.buf, this.labelNames);
        }
        else {
            this.buf.append(" : unknown\n");
        }
        this.text.add(this.buf.toString());
    }
    
    @Override
    public void visitCode() {
    }
    
    @Override
    public void visitFrame(int n, int i, Object[] array, int n2, Object[] array2) {
        this.buf.setLength(0);
        this.buf.append(this.ltab);
        this.buf.append("FRAME ");
        switch (n) {
            case -1:
            case 0: {
                this.buf.append("FULL [");
                this.appendFrameTypes(i, array);
                this.buf.append("] [");
                this.appendFrameTypes(n2, array2);
                this.buf.append(']');
                break;
            }
            case 1: {
                this.buf.append("APPEND [");
                this.appendFrameTypes(i, array);
                this.buf.append(']');
                break;
            }
            case 2: {
                this.buf.append("CHOP ").append(i);
                break;
            }
            case 3: {
                this.buf.append("SAME");
                break;
            }
            case 4: {
                this.buf.append("SAME1 ");
                this.appendFrameTypes(1, array2);
                break;
            }
        }
        this.buf.append('\n');
        this.text.add(this.buf.toString());
    }
    
    @Override
    public void visitInsn(int n) {
        this.buf.setLength(0);
        this.buf.append(this.tab2).append(Textifier.OPCODES[n]).append('\n');
        this.text.add(this.buf.toString());
    }
    
    @Override
    public void visitIntInsn(int n, int i) {
        this.buf.setLength(0);
        this.buf.append(this.tab2).append(Textifier.OPCODES[n]).append(' ').append((n == 188) ? Textifier.TYPES[i] : Integer.toString(i)).append('\n');
        this.text.add(this.buf.toString());
    }
    
    @Override
    public void visitVarInsn(int n, int i) {
        this.buf.setLength(0);
        this.buf.append(this.tab2).append(Textifier.OPCODES[n]).append(' ').append(i).append('\n');
        this.text.add(this.buf.toString());
    }
    
    @Override
    public void visitTypeInsn(int n, String s) {
        this.buf.setLength(0);
        this.buf.append(this.tab2).append(Textifier.OPCODES[n]).append(' ');
        this.appendDescriptor(0, s);
        this.buf.append('\n');
        this.text.add(this.buf.toString());
    }
    
    @Override
    public void visitFieldInsn(int n, String s, String str, String s2) {
        this.buf.setLength(0);
        this.buf.append(this.tab2).append(Textifier.OPCODES[n]).append(' ');
        this.appendDescriptor(0, s);
        this.buf.append('.').append(str).append(" : ");
        this.appendDescriptor(1, s2);
        this.buf.append('\n');
        this.text.add(this.buf.toString());
    }
    
    @Deprecated
    @Override
    public void visitMethodInsn(int n, String s, String s2, String s3) {
        if (this.api >= 327680) {
            super.visitMethodInsn(n, s, s2, s3);
            return;
        }
        this.doVisitMethodInsn(n, s, s2, s3, n == 185);
        return;
    }
    
    @Override
    public void visitMethodInsn(int n, String s, String s2, String s3, boolean b) {
        if (this.api < 327680) {
            super.visitMethodInsn(n, s, s2, s3, b);
            return;
        }
        this.doVisitMethodInsn(n, s, s2, s3, b);
        return;
    }
    
    private void doVisitMethodInsn(int n, String s, String str, String s2, boolean b) {
        this.buf.setLength(0);
        this.buf.append(this.tab2).append(Textifier.OPCODES[n]).append(' ');
        this.appendDescriptor(0, s);
        this.buf.append('.').append(str).append(' ');
        this.appendDescriptor(3, s2);
        this.buf.append('\n');
        this.text.add(this.buf.toString());
    }
    
    @Override
    public void visitInvokeDynamicInsn(String str, String s, Handle handle, Object... array) {
        this.buf.setLength(0);
        this.buf.append(this.tab2).append("INVOKEDYNAMIC").append(' ');
        this.buf.append(str);
        this.appendDescriptor(3, s);
        this.buf.append(" [");
        this.buf.append('\n');
        this.buf.append(this.tab3);
        this.appendHandle(handle);
        this.buf.append('\n');
        this.buf.append(this.tab3).append("// arguments:");
        if (array.length == 0) {
            this.buf.append(" none");
        }
        else {
            this.buf.append('\n');
            for (int i = 0; i < array.length; ++i) {
                this.buf.append(this.tab3);
                Object obj = array[i];
                if ((obj instanceof String)) {
                    Printer.appendString(this.buf, (String)obj);
                }
                else if ((obj instanceof Type)) {
                    Type type = (Type)obj;
                    if (type.getSort() == 11) {
                        this.appendDescriptor(3, type.getDescriptor());
                    }
                    else {
                        this.buf.append(type.getDescriptor()).append(".class");
                    }
                }
                else if ((obj instanceof Handle)) {
                    this.appendHandle((Handle)obj);
                }
                else {
                    this.buf.append(obj);
                }
                this.buf.append(", \n");
            }
            this.buf.setLength(this.buf.length() - 3);
        }
        this.buf.append('\n');
        this.buf.append(this.tab2).append("]\n");
        this.text.add(this.buf.toString());
    }
    
    @Override
    public void visitJumpInsn(int n, Label label) {
        this.buf.setLength(0);
        this.buf.append(this.tab2).append(Textifier.OPCODES[n]).append(' ');
        this.appendLabel(label);
        this.buf.append('\n');
        this.text.add(this.buf.toString());
    }
    
    @Override
    public void visitLabel(Label label) {
        this.buf.setLength(0);
        this.buf.append(this.ltab);
        this.appendLabel(label);
        this.buf.append('\n');
        this.text.add(this.buf.toString());
    }
    
    @Override
    public void visitLdcInsn(Object obj) {
        this.buf.setLength(0);
        this.buf.append(this.tab2).append("LDC ");
        if ((obj instanceof String)) {
            Printer.appendString(this.buf, (String)obj);
        }
        else if ((obj instanceof Type)) {
            this.buf.append(((Type)obj).getDescriptor()).append(".class");
        }
        else {
            this.buf.append(obj);
        }
        this.buf.append('\n');
        this.text.add(this.buf.toString());
    }
    
    @Override
    public void visitIincInsn(int i, int j) {
        this.buf.setLength(0);
        this.buf.append(this.tab2).append("IINC ").append(i).append(' ').append(j).append('\n');
        this.text.add(this.buf.toString());
    }
    
    @Override
    public void visitTableSwitchInsn(int n, int n2, Label label, Label... array) {
        this.buf.setLength(0);
        this.buf.append(this.tab2).append("TABLESWITCH\n");
        for (int i = 0; i < array.length; ++i) {
            this.buf.append(this.tab3).append(n + i).append(": ");
            this.appendLabel(array[i]);
            this.buf.append('\n');
        }
        this.buf.append(this.tab3).append("default: ");
        this.appendLabel(label);
        this.buf.append('\n');
        this.text.add(this.buf.toString());
    }
    
    @Override
    public void visitLookupSwitchInsn(Label label, int[] array, Label[] array2) {
        this.buf.setLength(0);
        this.buf.append(this.tab2).append("LOOKUPSWITCH\n");
        for (int i = 0; i < array2.length; ++i) {
            this.buf.append(this.tab3).append(array[i]).append(": ");
            this.appendLabel(array2[i]);
            this.buf.append('\n');
        }
        this.buf.append(this.tab3).append("default: ");
        this.appendLabel(label);
        this.buf.append('\n');
        this.text.add(this.buf.toString());
    }
    
    @Override
    public void visitMultiANewArrayInsn(String s, int i) {
        this.buf.setLength(0);
        this.buf.append(this.tab2).append("MULTIANEWARRAY ");
        this.appendDescriptor(1, s);
        this.buf.append(' ').append(i).append('\n');
        this.text.add(this.buf.toString());
    }
    
    @Override
    public Printer visitInsnAnnotation(int n, TypePath typePath, String s, boolean b) {
        return this.visitTypeAnnotation(n, typePath, s, b);
    }
    
    @Override
    public void visitTryCatchBlock(Label label, Label label2, Label label3, String s) {
        this.buf.setLength(0);
        this.buf.append(this.tab2).append("TRYCATCHBLOCK ");
        this.appendLabel(label);
        this.buf.append(' ');
        this.appendLabel(label2);
        this.buf.append(' ');
        this.appendLabel(label3);
        this.buf.append(' ');
        this.appendDescriptor(0, s);
        this.buf.append('\n');
        this.text.add(this.buf.toString());
    }
    
    @Override
    public Printer visitTryCatchAnnotation(int n, TypePath obj, String s, boolean b) {
        this.buf.setLength(0);
        this.buf.append(this.tab2).append("TRYCATCHBLOCK @");
        this.appendDescriptor(1, s);
        this.buf.append('(');
        this.text.add(this.buf.toString());
        Textifier textifier = this.createTextifier();
        this.text.add(textifier.getText());
        this.buf.setLength(0);
        this.buf.append(") : ");
        this.appendTypeReference(n);
        this.buf.append(", ").append(obj);
        this.buf.append(b ? "\n" : " // invisible\n");
        this.text.add(this.buf.toString());
        return textifier;
    }
    
    @Override
    public void visitLocalVariable(String str, String s, String signature, Label label, Label label2, int i) {
        this.buf.setLength(0);
        this.buf.append(this.tab2).append("LOCALVARIABLE ").append(str).append(' ');
        this.appendDescriptor(1, s);
        this.buf.append(' ');
        this.appendLabel(label);
        this.buf.append(' ');
        this.appendLabel(label2);
        this.buf.append(' ').append(i).append('\n');
        if (signature != null) {
            this.buf.append(this.tab2);
            this.appendDescriptor(2, signature);
            TraceSignatureVisitor signatureVisitor = new TraceSignatureVisitor(0);
            new SignatureReader(signature).acceptType(signatureVisitor);
            this.buf.append(this.tab2).append("// declaration: ").append(signatureVisitor.getDeclaration()).append('\n');
        }
        this.text.add(this.buf.toString());
    }
    
    @Override
    public Printer visitLocalVariableAnnotation(int n, TypePath obj, Label[] array, Label[] array2, int[] array3, String s, boolean b) {
        this.buf.setLength(0);
        this.buf.append(this.tab2).append("LOCALVARIABLE @");
        this.appendDescriptor(1, s);
        this.buf.append('(');
        this.text.add(this.buf.toString());
        Textifier textifier = this.createTextifier();
        this.text.add(textifier.getText());
        this.buf.setLength(0);
        this.buf.append(") : ");
        this.appendTypeReference(n);
        this.buf.append(", ").append(obj);
        for (int i = 0; i < array.length; ++i) {
            this.buf.append(" [ ");
            this.appendLabel(array[i]);
            this.buf.append(" - ");
            this.appendLabel(array2[i]);
            this.buf.append(" - ").append(array3[i]).append(" ]");
        }
        this.buf.append(b ? "\n" : " // invisible\n");
        this.text.add(this.buf.toString());
        return textifier;
    }
    
    @Override
    public void visitLineNumber(int i, Label label) {
        this.buf.setLength(0);
        this.buf.append(this.tab2).append("LINENUMBER ").append(i).append(' ');
        this.appendLabel(label);
        this.buf.append('\n');
        this.text.add(this.buf.toString());
    }
    
    @Override
    public void visitMaxs(int i, int j) {
        this.buf.setLength(0);
        this.buf.append(this.tab2).append("MAXSTACK = ").append(i).append('\n');
        this.text.add(this.buf.toString());
        this.buf.setLength(0);
        this.buf.append(this.tab2).append("MAXLOCALS = ").append(j).append('\n');
        this.text.add(this.buf.toString());
    }
    
    @Override
    public void visitMethodEnd() {
    }
    
    public Textifier visitAnnotation(String s, boolean b) {
        this.buf.setLength(0);
        this.buf.append(this.tab).append('@');
        this.appendDescriptor(1, s);
        this.buf.append('(');
        this.text.add(this.buf.toString());
        Textifier textifier = this.createTextifier();
        this.text.add(textifier.getText());
        this.text.add(b ? ")\n" : ") // invisible\n");
        return textifier;
    }
    
    public Textifier visitTypeAnnotation(int n, TypePath obj, String s, boolean b) {
        this.buf.setLength(0);
        this.buf.append(this.tab).append('@');
        this.appendDescriptor(1, s);
        this.buf.append('(');
        this.text.add(this.buf.toString());
        Textifier textifier = this.createTextifier();
        this.text.add(textifier.getText());
        this.buf.setLength(0);
        this.buf.append(") : ");
        this.appendTypeReference(n);
        this.buf.append(", ").append(obj);
        this.buf.append(b ? "\n" : " // invisible\n");
        this.text.add(this.buf.toString());
        return textifier;
    }
    
    public void visitAttribute(Attribute attribute) {
        this.buf.setLength(0);
        this.buf.append(this.tab).append("ATTRIBUTE ");
        this.appendDescriptor(-1, attribute.type);
        if ((attribute instanceof Textifiable)) {
            ((Textifiable)attribute).textify(this.buf, null);
        }
        else {
            this.buf.append(" : unknown\n");
        }
        this.text.add(this.buf.toString());
    }
    
    protected Textifier createTextifier() {
        return new Textifier();
    }
    
    protected void appendDescriptor(int n, String s) {
        if (n == 5 || n == 2 || n == 4) {
            if (s != null) {
                this.buf.append("// signature ").append(s).append('\n');
            }
        }
        else {
            this.buf.append(s);
        }
    }
    
    protected void appendLabel(Label label) {
        if (this.labelNames == null) {
            this.labelNames = new HashMap();
        }
        String string = (String)this.labelNames.get(label);
        if (string == null) {
            string = "L" + this.labelNames.size();
            this.labelNames.put(label, string);
        }
        this.buf.append(string);
    }
    
    protected void appendHandle(Handle handle) {
        int tag = handle.getTag();
        this.buf.append("// handle kind 0x").append(Integer.toHexString(tag)).append(" : ");
        boolean n = false;
        switch (tag) {
            case 1: {
                this.buf.append("GETFIELD");
                break;
            }
            case 2: {
                this.buf.append("GETSTATIC");
                break;
            }
            case 3: {
                this.buf.append("PUTFIELD");
                break;
            }
            case 4: {
                this.buf.append("PUTSTATIC");
                break;
            }
            case 9: {
                this.buf.append("INVOKEINTERFACE");
                n = true;
                break;
            }
            case 7: {
                this.buf.append("INVOKESPECIAL");
                n = true;
                break;
            }
            case 6: {
                this.buf.append("INVOKESTATIC");
                n = true;
                break;
            }
            case 5: {
                this.buf.append("INVOKEVIRTUAL");
                n = true;
                break;
            }
            case 8: {
                this.buf.append("NEWINVOKESPECIAL");
                n = true;
                break;
            }
        }
        this.buf.append('\n');
        this.buf.append(this.tab3);
        this.appendDescriptor(0, handle.getOwner());
        this.buf.append('.');
        this.buf.append(handle.getName());
        if (n == 0) {
            this.buf.append('(');
        }
        this.appendDescriptor(9, handle.getDesc());
        if (n == 0) {
            this.buf.append(')');
        }
    }
    
    private void appendAccess(int n) {
        if ((n & 0x1) != 0x0) {
            this.buf.append("public ");
        }
        if ((n & 0x2) != 0x0) {
            this.buf.append("private ");
        }
        if ((n & 0x4) != 0x0) {
            this.buf.append("protected ");
        }
        if ((n & 0x10) != 0x0) {
            this.buf.append("final ");
        }
        if ((n & 0x8) != 0x0) {
            this.buf.append("static ");
        }
        if ((n & 0x20) != 0x0) {
            this.buf.append("synchronized ");
        }
        if ((n & 0x40) != 0x0) {
            this.buf.append("volatile ");
        }
        if ((n & 0x80) != 0x0) {
            this.buf.append("transient ");
        }
        if ((n & 0x400) != 0x0) {
            this.buf.append("abstract ");
        }
        if ((n & 0x800) != 0x0) {
            this.buf.append("strictfp ");
        }
        if ((n & 0x1000) != 0x0) {
            this.buf.append("synthetic ");
        }
        if ((n & 0x8000) != 0x0) {
            this.buf.append("mandated ");
        }
        if ((n & 0x4000) != 0x0) {
            this.buf.append("enum ");
        }
    }
    
    private void appendComa(int n) {
        if (n != 0) {
            this.buf.append(", ");
        }
    }
    
    private void appendTypeReference(int typeRef) {
        TypeReference typeReference = new TypeReference(typeRef);
        switch (typeReference.getSort()) {
            case 0: {
                this.buf.append("CLASS_TYPE_PARAMETER ").append(typeReference.getTypeParameterIndex());
                break;
            }
            case 1: {
                this.buf.append("METHOD_TYPE_PARAMETER ").append(typeReference.getTypeParameterIndex());
                break;
            }
            case 16: {
                this.buf.append("CLASS_EXTENDS ").append(typeReference.getSuperTypeIndex());
                break;
            }
            case 17: {
                this.buf.append("CLASS_TYPE_PARAMETER_BOUND ").append(typeReference.getTypeParameterIndex()).append(", ").append(typeReference.getTypeParameterBoundIndex());
                break;
            }
            case 18: {
                this.buf.append("METHOD_TYPE_PARAMETER_BOUND ").append(typeReference.getTypeParameterIndex()).append(", ").append(typeReference.getTypeParameterBoundIndex());
                break;
            }
            case 19: {
                this.buf.append("FIELD");
                break;
            }
            case 20: {
                this.buf.append("METHOD_RETURN");
                break;
            }
            case 21: {
                this.buf.append("METHOD_RECEIVER");
                break;
            }
            case 22: {
                this.buf.append("METHOD_FORMAL_PARAMETER ").append(typeReference.getFormalParameterIndex());
                break;
            }
            case 23: {
                this.buf.append("THROWS ").append(typeReference.getExceptionIndex());
                break;
            }
            case 64: {
                this.buf.append("LOCAL_VARIABLE");
                break;
            }
            case 65: {
                this.buf.append("RESOURCE_VARIABLE");
                break;
            }
            case 66: {
                this.buf.append("EXCEPTION_PARAMETER ").append(typeReference.getTryCatchBlockIndex());
                break;
            }
            case 67: {
                this.buf.append("INSTANCEOF");
                break;
            }
            case 68: {
                this.buf.append("NEW");
                break;
            }
            case 69: {
                this.buf.append("CONSTRUCTOR_REFERENCE");
                break;
            }
            case 70: {
                this.buf.append("METHOD_REFERENCE");
                break;
            }
            case 71: {
                this.buf.append("CAST ").append(typeReference.getTypeArgumentIndex());
                break;
            }
            case 72: {
                this.buf.append("CONSTRUCTOR_INVOCATION_TYPE_ARGUMENT ").append(typeReference.getTypeArgumentIndex());
                break;
            }
            case 73: {
                this.buf.append("METHOD_INVOCATION_TYPE_ARGUMENT ").append(typeReference.getTypeArgumentIndex());
                break;
            }
            case 74: {
                this.buf.append("CONSTRUCTOR_REFERENCE_TYPE_ARGUMENT ").append(typeReference.getTypeArgumentIndex());
                break;
            }
            case 75: {
                this.buf.append("METHOD_REFERENCE_TYPE_ARGUMENT ").append(typeReference.getTypeArgumentIndex());
                break;
            }
        }
    }
    
    private void appendFrameTypes(int n, Object[] array) {
        for (int i = 0; i < n; ++i) {
            if (i > 0) {
                this.buf.append(' ');
            }
            if ((array[i] instanceof String)) {
                String s = (String)array[i];
                if (s.startsWith("[")) {
                    this.appendDescriptor(1, s);
                }
                else {
                    this.appendDescriptor(0, s);
                }
            }
            else if ((array[i] instanceof Integer)) {
                switch ((int)array[i]) {
                    case 0: {
                        this.appendDescriptor(1, "T");
                        break;
                    }
                    case 1: {
                        this.appendDescriptor(1, "I");
                        break;
                    }
                    case 2: {
                        this.appendDescriptor(1, "F");
                        break;
                    }
                    case 3: {
                        this.appendDescriptor(1, "D");
                        break;
                    }
                    case 4: {
                        this.appendDescriptor(1, "J");
                        break;
                    }
                    case 5: {
                        this.appendDescriptor(1, "N");
                        break;
                    }
                    case 6: {
                        this.appendDescriptor(1, "U");
                        break;
                    }
                }
            }
            else {
                this.appendLabel((Label)array[i]);
            }
        }
    }
}
